from .conda_env_utils import prepare_conda_env
from .initialize_repo import initialize_repo, clone
from .repositories import ProjectRepo, JupyterInterfaceRepo
from .configuration_options import Options
from .wrapper import tracks_results

__version__ = "0.0.41"



import numpy as np

from cadetrdm import Options


def test_options_hash():
    opt = Options()
    opt["array"] = np.linspace(2, 200)
    opt["nested_dict"] = {"ba": "foo", "bb": "bar"}
    initial_hash = opt.get_hash()
    s = opt.dumps()
    opt_recovered = Options.loads(s)
    post_serialization_hash = opt_recovered.get_hash()
    assert initial_hash == post_serialization_hash
    assert opt == opt_recovered


def test_options_file_io():
    opt = Options()
    opt["array"] = np.linspace(0, 2, 200)
    opt["nested_dict"] = {"ba": "foo", "bb": "bar"}
    initial_hash = opt.get_hash()
    opt.dump_json_file("options.json")
    opt_recovered = Options.load_json_file("options.json")
    post_serialization_hash = opt_recovered.get_hash()
    assert initial_hash == post_serialization_hash
    assert opt == opt_recovered


def test_excluded_keys():
    opt1 = Options()
    opt1["nested_dict"] = {"ba": "foo", "bb": "bar"}
    opt1.answer = 42
    opt1._n_cores = 9001

    opt2 = Options()
    opt2["nested_dict"] = {"ba": "foo", "bb": "bar"}
    opt2.answer = 42
    opt2._n_cores = 1

    assert opt1 == opt2



import importlib
import os
import sys
from pathlib import Path

from cadetrdm import clone, Options, ProjectRepo


class Study(ProjectRepo):
    def __init__(self, path, url, branch="main", name=None, *args, **kwargs):
        if name is None:
            self.name = Path(path).parts[-1]
        else:
            self.name = name
        self.url = url

        try:
            if not path.exists():
                clone(self.url, path)
            ProjectRepo(path).checkout(branch)
        except Exception as e:
            print(f"Error processing study {self.name}: {e}")
            return

        super().__init__(path, *args, **kwargs)

    @property
    def module(self):
        cur_dir = os.getcwd()

        os.chdir(self.path)
        sys.path.append(str(self.path))
        module = importlib.import_module(self.name)

        sys.path.remove(str(self.path))
        os.chdir(cur_dir)
        return module


class Case:
    def __init__(self, study: Study, options: Options, name: str = None):
        if name is None:
            name = options.get_hash()

        self.name = name
        self.study = study
        self.options = options
        self._options_hash = options.get_hash()
        self._results_branch = None

    @property
    def status_file(self):
        return Path(self.study.path).parent / (Path(self.study.path).name + ".status")

    @property
    def status(self):
        status, _ = self._read_status()
        return status

    @status.setter
    def status(self, status):
        """Update the status file with the current execution status."""

        with open(self.status_file, "w") as f:
            f.write(f"{status}@{self.study.current_commit_hash}")

    @property
    def status_hash(self):
        _, status_hash = self._read_status()
        return status_hash

    def _read_status(self):
        """Check the status of the study and decide whether to proceed.

        Args:
            repo_path (Path): The path to the repository containing the status file.

        Returns:
            tuple: A tuple containing the status string and the current hash,
            or None, None if the status cannot be determined.
        """

        if not self.status_file.exists():
            return None, None

        with open(self.status_file) as f:
            status = f.read().strip()
            try:
                status, current_hash = status.split("@")
            except ValueError as e:
                if status == '':
                    return None, None
                else:
                    raise e

            return status, current_hash

    @property
    def is_running(self, ):
        if self.status == 'running':
            return True

        return False

    @property
    def has_results_for_this_run(self):
        if self.results_branch is None:
            return False
        else:
            return True

    def _get_results_branch(self):
        output_log = self.study.output_log
        study_options_hash = self.options.get_hash()
        study_commit_hash = self.study.current_commit_hash
        has_been_run_with_these_options = False
        has_been_run_with_this_commit = False
        semi_correct_hits = []
        for log_entry in output_log:
            entry_options_hash = log_entry.options_hash
            entry_commit_hash = log_entry.project_repo_commit_hash
            if study_commit_hash == entry_commit_hash and study_options_hash == entry_options_hash:
                return log_entry.output_repo_branch
            elif study_commit_hash == entry_commit_hash and study_options_hash != entry_options_hash:
                has_been_run_with_this_commit = True
                semi_correct_hits.append(
                    f"Found matching study commit hash {study_commit_hash[:7]}, but incorrect options hash "
                    f"(needs: {study_options_hash[:7]}, has: {entry_options_hash[:7]})"
                )
            elif study_commit_hash != entry_commit_hash and study_options_hash == entry_options_hash:
                has_been_run_with_these_options = True
                semi_correct_hits.append(
                    f"Found matching options hash  {study_options_hash[:7]}, but incorrect study commit hash "
                    f"(needs: {study_commit_hash[:7]}, has: {entry_commit_hash[:7]})"
                )
        if has_been_run_with_these_options:
            [print(line) for line in semi_correct_hits]
            print(
                "No matching results were found for this study version, but results with these options were found for "
                "other study versions. Did you recently update the study?"
            )
        elif has_been_run_with_this_commit:
            [print(line) for line in semi_correct_hits]
            print(
                "No matching results were found for these options, but results with other options were found for "
                "this study versions. Did you recently change the options?"
            )
        else:
            print("No matching results were found for these options and study version.")
        return None

    def run_study(self, force=False):
        """Run specified study commands in the given repository."""
        if self.is_running and not force:
            print(f"{self.study.name} is currently running. Skipping...")
            return

        print(f"Running {self.name} in {self.study.path} with: {self.options}")
        if not self.options.debug:
            self.study.update()
        else:
            print("WARNING: Not updating the repositories while in debug mode.")

        if self.has_results_for_this_run and not force:
            print(f"{self.study.path} has already been computed with these options. Skipping...")
            return

        try:
            self.status = 'running'

            self.study.module.main(self.options, str(self.study.path))

            print("Command execution successful.")
            self.status = 'finished'

        except (KeyboardInterrupt, Exception) as e:
            print(f"Command execution failed: {e}")
            self.status = 'failed'
            return

    @property
    def _results_path(self):
        if self._results_branch is None:
            return None
        else:
            return self.study.path / (self.study._output_folder + "_cached") / self._results_branch

    def load(self, ):
        if self._results_branch is None or self.options.get_hash() != self._options_hash:
            self._results_branch = self._get_results_branch()
            self._options_hash = self.options.get_hash()

        if self._results_branch is None:
            print(f"No results available for Case({self.study.path, self.options.get_hash()[:7]})")
            return None

        if self._results_path.exists():
            return

        self.study.copy_data_to_cache(self._results_branch)

    @property
    def results_path(self):
        self.load()

        return self._results_path



import os
import random
from pathlib import Path

import git
import numpy as np
import pytest

from cadetrdm import initialize_repo, ProjectRepo, clone, Options
from cadetrdm.initialize_repo import init_lfs
from cadetrdm.io_utils import delete_path
from cadetrdm.repositories import OutputRepo, BaseRepo
from cadetrdm.wrapper import tracks_results


@pytest.fixture(scope="module")
def path_to_repo():
    # a "fixture" serves up shared, ready variables to test functions that should use the fixture as a kwarg
    return Path("test_repo")


def modify_code(path_to_repo):
    # Add changes to the project code
    random_number = random.randint(0, 265)
    filepath = path_to_repo / f"print_random_number.py"
    with open(filepath, "w") as file:
        file.write(f"print({random_number})\n")


def count_commit_number(repo):
    commit_log = repo._git.log("--oneline").split("\n")
    current_commit_number = len(commit_log)
    return current_commit_number


def example_generate_results_array(path_to_repo, output_folder):
    results_array = np.random.random((500, 3))
    np.savetxt(path_to_repo / output_folder / "result.csv", results_array, delimiter=",")
    return results_array


@tracks_results
def example_generate_results_with_options(repo, options):
    results_array = np.random.random((500, 3))
    np.savetxt(repo.output_path / "result.csv", results_array, delimiter=",")
    return results_array


def try_init_gitpython_repo(repo_path):
    Path(repo_path).exists()
    git.Repo(repo_path)
    return True


def try_initialize_git_repo(path_to_repo):
    if path_to_repo.exists():
        delete_path(path_to_repo)

    initialize_repo(path_to_repo, "results")

    assert try_init_gitpython_repo(path_to_repo)
    assert try_init_gitpython_repo(path_to_repo / "results")


def try_commit_code(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    current_commit_number = count_commit_number(repo)

    modify_code(path_to_repo)
    repo.commit("add code to print random number", add_all=True)

    updated_commit_number = count_commit_number(repo)
    assert current_commit_number + 1 == updated_commit_number


def try_commit_code_without_code_changes(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    current_commit_number = count_commit_number(repo)
    repo.commit("This commit will not be made", add_all=True)
    updated_commit_number = count_commit_number(repo)
    assert current_commit_number == updated_commit_number


def try_commit_results_data(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    current_commit_number = count_commit_number(repo.output_repo)
    with repo.track_results(results_commit_message="Add array"):
        example_generate_results_array(path_to_repo, output_folder=repo.output_path)
    updated_commit_number = count_commit_number(repo.output_repo)
    assert current_commit_number <= updated_commit_number
    return str(repo.output_repo.active_branch)


def try_commit_results_with_options(path_to_repo):
    options = Options()
    options.commit_message = "test with options"
    options.debug = False
    options.example = "123"
    options.dont_push = True

    repo = ProjectRepo(path_to_repo)
    current_commit_number = count_commit_number(repo.output_repo)

    example_generate_results_with_options(options, repo_path=repo.path)

    updated_commit_number = count_commit_number(repo.output_repo)
    assert current_commit_number <= updated_commit_number

    return str(repo.output_repo.active_branch)


def try_output_function(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    filepath = repo.output_path / "foo" / "bar"
    assert isinstance(filepath, Path)
    filepath = repo.output_data("foo/bar")
    assert isinstance(filepath, Path)


def try_print_log(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    repo.print_output_log()


def try_commit_results_with_uncommitted_code_changes(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    modify_code(path_to_repo)
    with pytest.raises(Exception):
        with repo.track_results(results_commit_message="Add array"):
            example_generate_results_array(path_to_repo, output_folder=repo.output_path)
    repo.commit("add code to print random number", add_all=True)


def try_load_previous_output(path_to_repo, branch_name):
    repo = ProjectRepo(path_to_repo)
    with repo.track_results(results_commit_message="Load array and extend"):
        cached_array_path = repo.input_data(branch_name=branch_name,
                                            file_path="result.csv")
        previous_array = np.loadtxt(cached_array_path, delimiter=",")
        extended_array = np.concatenate([previous_array, previous_array])
        extended_array_file_path = path_to_repo / repo.output_path / "extended_result.csv"
        np.savetxt(extended_array_file_path,
                   extended_array,
                   delimiter=",")
        assert cached_array_path.exists()
        assert extended_array_file_path.exists()


def try_add_remote(path_to_repo):
    repo = ProjectRepo(path_to_repo)
    repo.add_remote("git@jugit.fz-juelich.de:IBG-1/ModSim/cadet/CADET-RDM.git")
    assert "origin" in repo._git_repo.remotes


def try_initialize_from_remote():
    if Path("test_repo_from_remote").exists():
        delete_path("test_repo_from_remote")
    clone("https://jugit.fz-juelich.de/IBG-1/ModSim/cadet/rdm-examples-fraunhofer-ime-aachen",
          "test_repo_from_remote")
    assert try_init_gitpython_repo("test_repo_from_remote")


def test_init_over_existing_repo(monkeypatch):
    path_to_repo = Path("test_repo_2")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    os.chdir(path_to_repo)
    os.system(f"git init")
    with open("README.md", "w") as handle:
        handle.write("Readme-line 1\n")
    with open(".gitignore", "w") as handle:
        handle.write("foo.bar.*")
    repo = git.Repo(".")
    repo.git.add(".")
    repo.git.commit("-m", "Initial commit")
    os.chdir("..")

    # using monkeypath to simulate user input
    monkeypatch.setattr('builtins.input', lambda x: "Y")

    initialize_repo(path_to_repo)
    delete_path(path_to_repo)


def test_init_over_existing_code(monkeypatch):
    path_to_repo = Path("test_repo_2")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)

    with open(path_to_repo / "foobar.py", "w") as handle:
        handle.write("print('Hello World')\n")
    with open(path_to_repo / ".gitignore", "w") as handle:
        handle.write("foo.bar.*")

    # using monkeypath to simulate user input
    monkeypatch.setattr('builtins.input', lambda x: "Y")

    initialize_repo(path_to_repo)
    repo = git.Repo(path_to_repo)
    assert "Untracked files:" in repo.git.status()

    delete_path(path_to_repo)


def test_cache_with_non_rdm_repo(monkeypatch):
    path_to_repo = Path("non_rdm_repo")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    os.chdir(path_to_repo)
    os.system(f"git init")
    with open("README.md", "w") as handle:
        handle.write("Readme-line 1\n")
    with open(".gitignore", "w") as handle:
        handle.write("foo.bar.*")
    repo = git.Repo(".")
    repo.git.add(".")
    repo.git.commit("-m", "Initial commit")

    os.chdir("..")
    if Path("test_repo_non_rdm_imports").exists():
        delete_path("test_repo_non_rdm_imports")
    initialize_repo("test_repo_non_rdm_imports")
    os.chdir("test_repo_non_rdm_imports")

    repo = BaseRepo(".")

    if Path("external_cache/non_rdm_repo").exists():
        delete_path("external_cache/non_rdm_repo")
    if Path("foo/bar/non_rdm_repo").exists():
        delete_path("foo/bar/non_rdm_repo")
    # import two repos and confirm verify works.
    repo.import_remote_repo(source_repo_location=".." / path_to_repo, source_repo_branch="master")
    repo.import_remote_repo(source_repo_location=".." / path_to_repo, source_repo_branch="master",
                            target_repo_location="foo/bar/non_rdm_repo")
    repo.verify_unchanged_cache()


def test_add_lfs_filetype():
    path_to_repo = Path("test_repo_3")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    initialize_repo(path_to_repo)
    file_type = "*.bak"
    init_lfs(lfs_filetypes=[file_type], path=path_to_repo)
    repo = ProjectRepo(path_to_repo)
    repo.add_all_files()
    repo.commit(f"Add {file_type} to lfs")
    delete_path(path_to_repo)


def test_error_stack():
    path_to_repo = Path("test_repo_3")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    initialize_repo(path_to_repo)
    repo = ProjectRepo(path_to_repo)
    try:
        with repo.track_results("test error"):
            raise ValueError("This is an error message with \n a line break")
    except ValueError:
        pass
    with open("test_repo_3/output/error.stack", "r") as handle:
        lines = handle.readlines()

    error_line = '    raise ValueError("This is an error message with \\n a line break")\n'
    assert error_line in lines


def test_cadet_rdm(path_to_repo):
    # because these depend on one-another and there is no native support afaik for sequential tests
    # these tests are called sequentially here as try_ functions.
    try_initialize_git_repo(path_to_repo)
    # try_initialize_from_remote()

    try_add_remote(path_to_repo)
    # try_add_submodule(path_to_repo)
    try_commit_code(path_to_repo)
    try_commit_code_without_code_changes(path_to_repo)
    try_commit_results_with_uncommitted_code_changes(path_to_repo)
    try_output_function(path_to_repo)

    try_commit_results_with_options(path_to_repo)
    results_branch_name = try_commit_results_data(path_to_repo)
    try_print_log(path_to_repo)

    try_commit_code(path_to_repo)

    try_load_previous_output(path_to_repo, results_branch_name)


def test_with_detached_head():
    path_to_repo = Path("test_repo_2")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    initialize_repo(path_to_repo)
    os.chdir(path_to_repo)
    for i in range(2):
        with open("README.md", "a") as handle:
            handle.write(f"Readme-line {i}\n")
        os.system(f"git add .")
        os.system(f"git commit -m foobar{i}")

    os.system("git checkout HEAD~1")
    repo = ProjectRepo(".")
    with repo.track_results("foo"):
        pass

    os.chdir("..")


def test_with_external_repos():
    path_to_repo = Path("test_repo_external_data")
    if path_to_repo.exists():
        delete_path(path_to_repo)
    os.makedirs(path_to_repo)
    initialize_repo(path_to_repo)

    os.chdir(path_to_repo)

    # to be able to hand over a valid branch, I first need to determine that branch
    imported_repo = OutputRepo("../test_repo/results")
    branch_name = imported_repo.active_branch.name

    repo = ProjectRepo(".")

    # import two repos and confirm verify works.
    repo.import_remote_repo(source_repo_location="../test_repo/results", source_repo_branch=branch_name)
    repo.import_remote_repo(source_repo_location="../test_repo/results", source_repo_branch=branch_name,
                            target_repo_location="foo/bar/repo")
    # delete folder and reload
    delete_path("foo/bar/repo")

    with pytest.raises(Exception):
        repo.verify_unchanged_cache()

    repo.fill_data_from_cadet_rdm_json()
    repo.verify_unchanged_cache()

    # Test if re_load correctly re_loads by modifying and then reloading
    with open("external_cache/results/README.md", "w") as file_handle:
        file_handle.writelines(["a", "b", "c"])
    repo.fill_data_from_cadet_rdm_json(re_load=True)
    repo.verify_unchanged_cache()

    # modify file and confirm error raised
    with open("external_cache/results/README.md", "w") as file_handle:
        file_handle.writelines(["a", "b", "c"])
    with pytest.raises(Exception):
        repo.verify_unchanged_cache()

    os.chdir("..")



import pytest
from pathlib import Path
import random
import os

from click.testing import CliRunner

from cadetrdm.cli_integration import cli
from cadetrdm.io_utils import delete_path

runner = CliRunner()
if os.path.exists("test_repo_cli"):
    delete_path("test_repo_cli")

os.makedirs("test_repo_cli")
os.chdir("test_repo_cli")


def modify_code(path_to_repo):
    # Add changes to the project code
    random_number = random.randint(0, 265)
    filepath = Path(path_to_repo) / f"print_random_number.py"
    with open(filepath, "w") as file:
        file.write(
            f"print({random_number})\n"
            'with open("output/data.txt", "w") as handle:\n'
            f"    handle.write({random_number})\n"
        )


def test_01_initialize_repo():
    result = runner.invoke(cli, ["init", ])
    print(result.output)
    assert result.exit_code == 0


def test_02_add_remote():
    result = runner.invoke(cli, ["remote", "add", "https://jugit.fz-juelich.de/r.jaepel/API_test_project"])
    print(result.output)
    assert result.exit_code == 0
    os.chdir("output")
    result = runner.invoke(cli, ["remote", "add", "https://jugit.fz-juelich.de/r.jaepel/API_test_project_output"])
    print(result.output)
    os.chdir("..")
    assert result.exit_code == 0


@pytest.mark.server_api
def test_02b_clone():
    os.chdir("..")
    if os.path.exists("test_repo_cli_cloned"):
        delete_path("test_repo_cli_cloned")
    result = runner.invoke(cli, ["clone", "test_repo_cli", "test_repo_cli_cloned"])
    print(result.output)
    os.chdir("test_repo_cli")
    assert result.exit_code == 0


def test_03_commit_results_with_uncommited_code_changes():
    modify_code(".")

    result = runner.invoke(cli, ["run", "python", "print_random_number.py",
                                 "create data"])
    print(result.output)
    assert result.exit_code != 0


def test_04_commit_code():
    modify_code(".")

    result = runner.invoke(cli, ["commit", "-m", "add code", "-a"])
    print(result.output)
    assert result.exit_code == 0


# def test_05_commit_results():
#     result = runner.invoke(cli, ["commit", "-m", "add code", "-a"])
#     print(result.output)
#     assert result.exit_code == 0
#     result = runner.invoke(cli, ["run", "python", "print_random_number.py",
#                                  "create data"])
#     print(result.output)
#     assert result.exit_code == 0
#
#
# def test_05b_execute_command():
#     result = runner.invoke(cli, ["commit", "-m", "add code", "-a"])
#     print(result.output)
#     assert result.exit_code == 0
#     result = runner.invoke(cli, ["run", "command", "python", "print_random_number.py",
#                                  "create data"])
#     print(result.output)
#     assert result.exit_code == 0


def test_06_print_log():
    result = runner.invoke(cli, ["log"])
    print(result.output)
    assert result.exit_code == 0


def test_07_lfs_add():
    result = runner.invoke(cli, ["lfs", "add", "pptx"])
    print(result.output)
    assert result.exit_code == 0


def test_08_data_import():
    result = runner.invoke(cli,
                           ["data", "import", "https://github.com/ronald-jaepel/workshop_demo_output",
                            "2023-12-19_10-43-15_output_from_master_86541bc", "imported/repo/data"])
    print(result.output)
    assert result.exit_code == 0

# def test_09_data_verify():
#     with open()
#     result = runner.invoke(cli, ["data", "verify"])



from pathlib import Path

from tabulate import tabulate


class OutputLog:
    def __init__(self, filepath=None):
        # ToDo add classmethod for initialization from list of lists
        if not Path(filepath).exists():
            self._entry_list = [[], []]
            self.entries = []
            return

        self._entry_list = self._read_file(filepath)
        self.entries = self._entries_from_entry_list(self._entry_list)

    def __iter__(self):
        self._index = 0
        return self

    def __next__(self):
        try:
            entry = self.entries[self._index]
            self._index += 1
            return entry
        except IndexError:
            raise StopIteration

    def _entries_from_entry_list(self, entry_list):
        header = self._convert_header(entry_list[0])
        if len(header) < 9:
            header.append("options_hash")
        entry_list = entry_list[1:]
        entry_dictionaries = []
        for entry in entry_list:
            if len(entry) < len(header):
                entry += [""] * (len(header) - len(entry))

            entry_dictionaries.append(
                {key: value for key, value in zip(header, entry)}
            )
        return [LogEntry(**entry) for entry in entry_dictionaries]

    def _read_file(self, filepath):
        with open(filepath) as handle:
            lines = handle.readlines()
        lines = [line.replace("\n", "").split("\t") for line in lines]
        return lines

    def _convert_header(self, header):
        return [entry.lower().replace(" ", "_") for entry in header]

    def __str__(self):
        return tabulate(self._entry_list[1:], headers=self._entry_list[0])


class LogEntry:
    def __init__(self, output_repo_commit_message, output_repo_branch, output_repo_commit_hash,
                 project_repo_commit_hash, project_repo_folder_name, project_repo_remotes, python_sys_args, tags,
                 options_hash):
        self.output_repo_commit_message = output_repo_commit_message
        self.output_repo_branch = output_repo_branch
        self.output_repo_commit_hash = output_repo_commit_hash
        self.project_repo_commit_hash = project_repo_commit_hash
        self.project_repo_folder_name = project_repo_folder_name
        self.project_repo_remotes = project_repo_remotes
        self.python_sys_args = python_sys_args
        self.tags = tags
        self.options_hash = options_hash

    def __repr__(self):
        return f"OutputEntry('{self.output_repo_commit_message}', '{self.output_repo_branch}')"


if __name__ == '__main__':
    output_log = OutputLog(filepath=r"C:\Users\ronal\PycharmProjects\CADETRDM\tests\test_repo\results\log.tsv")
    print(output_log.entries)

    output_log = OutputLog(
        filepath=r"C:\Users\ronal\PycharmProjects\CADETRDM\tests\test_repo\results\lognonexistant.tsv")
    print(output_log.entries)



import os
import time
from datetime import datetime
from pathlib import Path

import pytest

from cadetrdm import Options
from cadetrdm.batch_runner import Study, Case
from cadetrdm.io_utils import delete_path


@pytest.mark.server_api
def test_module_import():
    WORK_DIR = Path.cwd() / "tmp"
    WORK_DIR.mkdir(parents=True, exist_ok=True)

    rdm_example = Study(
        WORK_DIR / 'template',
        "git@jugit.fz-juelich.de:r.jaepel/rdm_example.git",
    )

    assert hasattr(rdm_example.module, "main")
    assert hasattr(rdm_example.module, "setup_optimization_problem")

    delete_path(WORK_DIR)

#
# @pytest.mark.server_api
# def test_parallel_runner():
#     work_dir = Path.cwd() / "batch_repos"
#     if os.path.exists(work_dir):
#         delete_path(work_dir)
#     work_dir.mkdir(parents=True, exist_ok=True)
#
#     rdm_example = Study(
#         work_dir / 'template',
#         "git@jugit.fz-juelich.de:r.jaepel/rdm_example.git",
#     )
#
#     studies = [rdm_example]
#     push = True
#
#     DEFAULT_OPTIONS = [
#         Options({
#             'objective': 'single-objective',
#             'optimizer_options': {
#                 "optimizer": "U_NSGA3",
#                 "pop_size": 2,
#                 "n_cores": 1,
#                 "n_max_gen": 2,
#             },
#             "datetime": str(datetime.now()),
#             'debug': False,
#             'push': push,
#         }),
#         Options({
#             'objective': 'single-objective',
#             'optimizer_options': {
#                 "optimizer": "U_NSGA3",
#                 "pop_size": 3,
#                 "n_cores": 1,
#                 "n_max_gen": 2,
#             },
#             "datetime": str(datetime.now()),
#             'debug': False,
#             'push': push,
#         }),
#     ]
#
#     python_commands = []
#     cases = []
#     # Default cases
#     for study in studies:
#         for options in DEFAULT_OPTIONS:
#             options.commit_message = f"Trying new things."
#             if Case(study=study, options=options).has_results_for_this_run:
#                 continue
#
#             local_study = Study(
#                 path=study.path.parent / f"{study.path.name}_{options.get_hash()[:7]}",
#                 url=study.url,
#                 name=study.name
#             )
#             study_path = local_study.path.absolute()
#             options.dump_json_file(local_study.output_path / "options.json")
#             python_commands.append(
#                 f"python {study_path / study.name / 'cli.py'} --options {local_study.output_path / 'options.json'}"
#             )
#             time.sleep(1)
#             cases.append((local_study, options))
#
#     # for local_study, options in cases:
#     #     case = Case(local_study, options)
#     #     case.run_study(force=False)
#
#     # for line in python_commands:
#     #     print(line)
#     #     os.system(line)
